# -*- coding: utf-8 -*-
###
# Copyright (c) 2009, David Valdez
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions, and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions, and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
#   * Neither the name of the author of this software nor the name of
#     contributors to this software may be used to endorse or promote products
#     derived from this software without specific prior written consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
###

import urllib
import random
import simplejson

import supybot.utils as utils
from supybot.commands import *
import supybot.plugins as plugins
import supybot.ircutils as ircutils
import supybot.callbacks as callbacks

from supybot import ircmsgs

import re
splitTokensRE = re.compile('([A-Za-z0-9_`]+)')

# Here is the format of ChannelMap:
#channelMap = \
#    {
#    '#meetbot-test':   {('en','es'): ('#meetbot-test2'  ,)},
#    '#meetbot-test2':  {('es','en'): ('#meetbot-test'   ,)},
#    }
channelMap = { }
def addChannel(chanFrom, lang1, lang2, chanTo):
    channel_d = channelMap.setdefault(chanFrom, { })
    # I could use sets here...
    toChannels = channel_d.setdefault((lang1, lang2), [ ])
    if chanTo in toChannels: return False  # it's already in the list.
    toChannels.append(chanTo)
    return True
def delChannel(chanFrom, lang1, lang2, chanTo):
    if chanFrom not in channelMap: return False
    channel_d = channelMap[chanFrom]
    if (lang1, lang2) not in channel_d: return False
    toChannels = channel_d[(lang1, lang2)]
    if chanTo not in toChannels: return False
    toChannels.remove(chanTo)
    return True


def translate_google(lang1, lang2, text):
    text = text.replace('<','&lt;').replace('>','&gt;')
    params = {'q': text,'langpair': lang1 +'|' + lang2}
    params = urllib.urlencode(params)

    url = "http://ajax.googleapis.com/ajax/services/"+\
          "language/translate?v=1.0&"+params
    html = utils.web.getUrl(url)
    html = simplejson.loads(html)

    if html["responseStatus"] == 200:
        text = html['responseData']['translatedText']
        text = text.replace('&lt;','<').replace('&gt;','>').\
                   replace('&quot;','"').replace('&#39;',"'").\
                   replace('&amp;','&')
        return text
    else:
        print "No Server Response"
        return None
translate = translate_google


class gtranslate(callbacks.Plugin):
    """ """
    threaded = True
    def callCommand(self,command,irc,msg,*args,**kwargs):
        try:
            super(gtranslate,self).callCommand(command,irc,msg,*args,**kwargs)
        except utils.web.Error,e:
            irc.error(str(e))

    def gtranslate(self,irc,msg,args,lang1,lang2,string):
        """[<lang>] [<string>]
        
        Return a translate  string
        """
    
        if not string or  not lang1  or not lang2:
            irc.reply("gtranslate  {es-en} text to translate")
        params = {'q': string,'langpair': lang1 +'|' + lang2}
        params = urllib.urlencode(params)
        url = "http://ajax.googleapis.com/ajax/services/language/translate?v=1.0&"+params
        html = utils.web.getUrl(url)
        html = simplejson.loads(html)
        if(html["responseStatus"] == 200):
            html = html['responseData']['translatedText']
            irc.reply(html)
        else:
            irc.reply("Service temporaly unavalaible")
    gtranslate = wrap(gtranslate,['somethingWithoutSpaces','somethingWithoutSpaces','text'])

    def transadmin(self, irc, msg, args,
                  command, ArgFromChan, lang1, lang2, ArgToChan):
        """Add/Remove channels to join with translation (owner only).

        Subcommands:
        list  ;
        add <from> <lang1> <lang2> <to>  ;
        del <from> <lang1> <lang2> <to>
        """
        if command == 'list':
            maxLen = max(len(x) for x in channelMap)
            for chanFrom in sorted(channelMap):
                for lang1,lang2 in sorted(channelMap[chanFrom]):
                    for chanTo in channelMap[chanFrom][(lang1,lang2)]:
                        irc.reply("%s   %s %s   %s"%
                              (chanFrom.ljust(maxLen), lang1, lang2, chanTo))
        elif command == 'add':
            addChannel(ArgFromChan, lang1, lang2, ArgToChan)
        elif command == 'del':
            ret = delChannel(ArgFromChan, lang1, lang2, ArgToChan)
            if ret: irc.replySuccess()
            else: irc.reply("Failed - couldn't remove channels.")
        else:
            irc.reply("sub-commands: list ; add <from> <lang1> <lang2> <to> ; del <from> <lang1> <lang2> <to>")

    transadmin = wrap(transadmin,
                     ['owner',
                      'somethingWithoutSpaces',
                      optional('validChannel'),
                      optional('somethingWithoutSpaces'),
                      optional('somethingWithoutSpaces'),
                      optional('validChannel')])


    def doPrivmsg(self, irc, msg):
        nick = msg.nick
        channel = msg.args[0]
        payload = msg.args[1]
        payload = payload.strip(' \x01')
        action = False
        #from rkddp.interact import interact ; interact()
        if payload[:6] == 'ACTION':
            payload = payload[7:]
            action = True
        # provide a way to prevent auto-translating.
        if len(payload) > 0 and payload[0] in "!,":
            return

        # format:     <source_channel>: {}
        if channel not in channelMap:
            return
        print payload
        fromChanNicks = channelNicks(irc, channel)
        for (lang1, lang2), targetChannels in channelMap[channel].iteritems():
            # Do the actual translation.
            newText = translate(lang1, lang2, payload)
            # Send it to all the channels that should receive it:
            for targetChannel in targetChannels:
                # Mangling text to prevent highlighting.
                nicks = fromChanNicks & channelNicks(irc, targetChannel)
                
                newText = newText.encode('utf-8')
                print newText
                if action:  newMessage = "* *%s* %s"%(nick, newText)
                else:       newMessage = "<%s> %s"%(nick, newText)
                newMessage = mangleLine(newMessage, nicks)
                irc.sendMsg(ircmsgs.privmsg(targetChannel, newMessage))

    def doTopic(self, irc, msg):
        nick = msg.nick
        channel = msg.args[0]
        payload = msg.args[1]

        if len(payload) > 0 and payload[0] in "!,":
            return
        if channel not in channelMap:
            return
        print "TOPIC", payload
        fromChanNicks = channelNicks(irc, channel)
        for (lang1, lang2), targetChannels in channelMap[channel].iteritems():
            # Do the actual translation.
            newText = translate(lang1, lang2, payload)
            # Send it to all the channels that should receive it:
            for targetChannel in targetChannels:
                nicks = fromChanNicks & channelNicks(irc, targetChannel)
                print "TOPIC:", newText
                newText = newText.encode('utf-8')
                newMessage = "-!- %s changed the topic of %s to: %s"%\
                             (nick, channel, newText)
                newMessage = mangleLine(newMessage, nicks)
                irc.sendMsg(ircmsgs.privmsg(targetChannel, newMessage))



def channelNicks(irc, channel):
    return set( user.lower() for user in
                irc.state.channels[channel].users)
def mangleLine(line, nicks):
    return "".join( mangleToken(t,nicks) for t in
                       splitTokensRE.split(line) )

def mangleToken(token, nicks):
    """If it's a nick, mangle.  Otherwise do nothing."""
    if token.lower() in nicks: return mangleNick(token)
    else: return token
unicodeChars = u'аеіоАВЕКМОРТСХΑΒΕΖΗΙΚΜΝΟΡΤΥѕсух'
asciiChars   =  'aeioABEKMOPTCKABEZHIKMNOPTYscyx'
unicodeMapping = { }
for a, u in zip(asciiChars, unicodeChars):
    unicodeMapping[a] = u.encode('utf8')

def mangleNickSwap(token):  # swap last two letters
    if len(token) == 1: return token
    else: return "".join((token[:-2], token[-1], token[-2]))
def mangleNickSwap2(token):  # swap last two letters
    if len(token) == 1: return token
    elif len(token) <=3: return "".join((token[:-2], token[-1], token[-2]))
    else: return "".join((token[:-3], token[-2], token[-3], token[-1]))
def mangleNickAsterisk(token):  # make an asterisk
    if len(token) == 1: return token
    else: return "".join((token[:-2], '*', token[-1]))
def mangleNickUnicode(token):  # make an asterisk
    token = list(token)
    token.reverse()
    for i, t in enumerate(token):
        if t in unicodeMapping:
            token[i] = unicodeMapping[t]
        break
    token = "".join(reversed(token))
    return token
def mangleNickShuffle(token):  # make an asterisk
    if len(token) <= 3: return mangleNickSwap(token)
    first, middle, last = token[0], token[1:-1], token[-1]
    middle = [ x for x in middle ]
    random.shuffle(middle)
    newToken = first + ("".join(middle)) + last
    if newToken == token:
        print "trying mangleNickShuffle again:", token
        return mangleNickShuffle(token)
    return newToken
#mangleNick = mangleNickUnicode
#mangleNick = mangleNickShuffle
mangleNick = mangleNickSwap2



Class = gtranslate


   
